#include <stdio.h>
#include <conio.h>
#include <windows.h>
#include "KWindow.h"
#include "misc.h"

KWindow::KWindow(HANDLE hStdOut)
{
	hCon = hStdOut;
	curSelRow = 0;
	lpBgBuffer = NULL;
}

KWindow::~KWindow()
{
	if(lpBgBuffer) delete lpBgBuffer;
}

void KWindow::gotoxy(SHORT x, SHORT y)
{
	COORD coord;
	coord.X = x;
	coord.Y = y;
	SetConsoleCursorPosition(hCon,coord);
}


void KWindow::SetPos(short x,short y)
{
	posX = x;
	posY = y;
}

void KWindow::SetSize(short x,short y)
{
	sizeX = x;
	sizeY = y;
}

void KWindow::SetTC(short c)
{
	colorT = c;
}

void KWindow::SetBC(short c)
{
	colorB = c;
}

void KWindow::Init(char *text,short px,short py,short sx,short sy,short ct,short cb,BOOL shadowp)
{
	posX = px;
	posY = py;
	sizeX = sx;
	sizeY = sy;
	colorT = ct;
	colorB = cb;
	strcpy(label,text);
	shadow = shadowp;
}

void KWindow::SetLabel(char *text)
{
	strcpy(label,text);
}

void KWindow::SaveBG(void)
{
	
	CONSOLE_SCREEN_BUFFER_INFO ci;
	GetConsoleScreenBufferInfo(hCon,&ci);
	
	dwBufferSize.X = sizeX;
	dwBufferSize.Y = sizeY;
	dwBufferCoord.X = 0;
	dwBufferCoord.Y = 0;
	srReadRegion.Left = posX;
	srReadRegion.Top = posY;
	srReadRegion.Right = posX + sizeX;
	srReadRegion.Bottom = posY + sizeY;
	if(shadow)	
	{
		dwBufferSize.X ++;
		dwBufferSize.Y ++;
		srReadRegion.Right ++;
		srReadRegion.Bottom ++;
	}
	if(lpBgBuffer) delete lpBgBuffer;
    lpBgBuffer = new CHAR_INFO[dwBufferSize.X * dwBufferSize.Y * sizeof(CHAR_INFO)];
	kASSERT(lpBgBuffer!=NULL);
	
	ReadConsoleOutput(hCon,lpBgBuffer,dwBufferSize,dwBufferCoord,&srReadRegion);
}

void KWindow::RestoreBG(HANDLE hConsole)
{
	HANDLE h;
	
	if(!lpBgBuffer) return;
	if(hConsole == 0) h = hCon;
	else h = hConsole;
	WriteConsoleOutput(h,lpBgBuffer,dwBufferSize,dwBufferCoord,&srReadRegion);
}

void KWindow::DrawFrame(HANDLE hConsole)
{
	COORD dwWriteCoord;
	DWORD cCharsWritten;
	char tmp[STR_SIZE];

	dwWriteCoord.X = posX;
	dwWriteCoord.Y = posY;
	for(int i = 0; i < sizeY; i++)
	{
		dwWriteCoord.X = posX;
		dwWriteCoord.Y = posY + i;
		FillConsoleOutputCharacter(hConsole,g_FrameChar[0], 1,dwWriteCoord, &cCharsWritten);
		dwWriteCoord.X = posX + sizeX - 1;
		FillConsoleOutputCharacter(hConsole,g_FrameChar[0], 1,dwWriteCoord, &cCharsWritten);
	}
	dwWriteCoord.X = posX;
	dwWriteCoord.Y = posY;
	FillConsoleOutputCharacter(hConsole,g_FrameChar[1], sizeX,dwWriteCoord, &cCharsWritten);
	dwWriteCoord.X = posX;
	dwWriteCoord.Y = posY + sizeY - 1;
	FillConsoleOutputCharacter(hConsole,g_FrameChar[1], sizeX,dwWriteCoord, &cCharsWritten);

	dwWriteCoord.X = posX;
	dwWriteCoord.Y = posY;
	FillConsoleOutputCharacter(hConsole,g_FrameChar[2], 1,dwWriteCoord, &cCharsWritten);
	dwWriteCoord.X = posX + sizeX - 1;
	dwWriteCoord.Y = posY;
	FillConsoleOutputCharacter(hConsole,g_FrameChar[3], 1,dwWriteCoord, &cCharsWritten);
	dwWriteCoord.X = posX;
	dwWriteCoord.Y = posY + sizeY - 1;
	FillConsoleOutputCharacter(hConsole,g_FrameChar[4], 1,dwWriteCoord, &cCharsWritten);
	dwWriteCoord.X = posX + sizeX - 1;
	dwWriteCoord.Y = posY + sizeY - 1;
	FillConsoleOutputCharacter(hConsole,g_FrameChar[5], 1,dwWriteCoord, &cCharsWritten);

	dwWriteCoord.X = posX + 2;
	dwWriteCoord.Y = posY;
	sprintf(tmp," %s ",label);
	if(strlen(tmp) > (unsigned)(sizeX - 4))
	{
		strrev(tmp);
		tmp[(sizeX - 4) - 2] = '.';
		tmp[(sizeX - 4) - 1] = '.';
		tmp[(sizeX - 4)] = 0;
		strrev(tmp);
	}
	WriteConsoleOutputCharacter(hConsole,tmp,strlen(tmp),dwWriteCoord, &cCharsWritten);

}


void KWindow::Show(HANDLE hConsole)
{
	COORD	dwWriteCoord;
	DWORD	cCharsWritten;
	int		i;

	dwWriteCoord.X = posX;
	dwWriteCoord.Y = posY;
	for(i = 0; i < sizeY; i++)
	{
		dwWriteCoord.X = posX;
		dwWriteCoord.Y = posY + i;
		FillConsoleOutputCharacter(hConsole,' ', sizeX,dwWriteCoord, &cCharsWritten);
		FillConsoleOutputAttribute(hConsole,colorT|colorB, sizeX,dwWriteCoord, &cCharsWritten);
		dwWriteCoord.X = posX + sizeX - 1;
	}
	DrawFrame(hConsole);
	if(shadow)
	{
		WORD attr[80];
		ReadAttribute(hConsole,attr,posX + 1,posY + sizeY,sizeX);
		for(int i = 0; i< 79; i++)
		{
			attr[i] &= 15;
			if(attr[i]>7) attr[i] -= 7;
		}
		WriteAttribute(hConsole,attr,posX + 1,posY + sizeY,sizeX);
	
		for( i = posY+1; i < (posY+sizeY+1); i++)
		{
			ReadAttribute(hConsole,attr,posX + sizeX,i,1);
			attr[0] &= 15;
			if(attr[0]>7) attr[0] -= 7;
			WriteAttribute(hConsole,attr,posX + sizeX,i,1);
		}
	}
}


void KWindow::Hide()
{
	RestoreBG();
}

BOOL KWindow::WriteRow(HANDLE hConsole,int iRow,LPCTSTR lpRow,short tc,short bc)
{
	COORD dwWriteCoord;
	DWORD cCharsWritten;
	int size = (int)strlen(lpRow)>(sizeX-2)?(sizeX-2):(strlen(lpRow));

	dwWriteCoord.X = posX + 1;
	dwWriteCoord.Y = posY + 1 + iRow;
	WriteConsoleOutputCharacter(hConsole,lpRow, size,dwWriteCoord, &cCharsWritten);
	if( tc != -1 )
	{
		FillConsoleOutputAttribute(hConsole,
			((tc==-1?colorT:tc)|(bc==-1?colorB:bc)), 
			size,dwWriteCoord, &cCharsWritten);
	}
return TRUE;	
}


int KWindow::SetAttributes(int x,int y,int size,short tc,short bc)
{
	COORD dwWriteCoord;
	DWORD cCharsWritten;
	
	dwWriteCoord.X = x;
	dwWriteCoord.Y = y;
	FillConsoleOutputAttribute(hCon,tc|bc,size,dwWriteCoord,&cCharsWritten);
	return 0;
}

// select row in window
// return	0	= OK
//			-1	= up
//			1   = down
int KWindow::RowAttribute(int iRow,short tc,short bc)
{
	COORD dwWriteCoord;
	DWORD cCharsWritten;
	
	if(iRow > sizeY - 3) return 1;
	if(iRow < 0) return -1;
	dwWriteCoord.X = posX + 1;
	dwWriteCoord.Y = posY + 1 + iRow;
	FillConsoleOutputAttribute(hCon,
		((tc==-1?colorT:tc)|(bc==-1?colorB:bc)),
		sizeX-2,dwWriteCoord, &cCharsWritten);
	return 0;
}

// select row in window
// return	0	= OK
//			-1	= up
//			1   = down
int KWindow::RowAttributeCon(HANDLE hConsole,int iRow,short tc,short bc)
{
	COORD dwWriteCoord;
	DWORD cCharsWritten;
	
	if(iRow > sizeY - 3) return 1;
	if(iRow < 0) return -1;
	dwWriteCoord.X = posX + 1;
	dwWriteCoord.Y = posY + 1 + iRow;
	FillConsoleOutputAttribute(hConsole,
		((tc==-1?colorT:tc)|(bc==-1?colorB:bc)),
		sizeX-2,dwWriteCoord, &cCharsWritten);
	return 0;
}
